from distutils.core import setup

setup(
    name='athena-run',
    version='0.1.2dev',
    packages=['tracing','patch'],
    license='Creative Commons Attribution-Noncommercial-Share Alike license',
    long_description="ML Tracing Tools",
)
